import cupy as cp  # type: ignore
import numpy as np  # type: ignore
from gepapy.operations import Operations as Operations
from typing import Optional, Union

class Single_Machine(Operations):
    def __init__(
        self,
        processing_time: Optional[Union[list, np.ndarray, cp.core.core.ndarray]],
        due_date: Optional[Union[list, np.ndarray, cp.core.core.ndarray]],
        weights: Optional[Union[list, np.ndarray, cp.core.core.ndarray]],
        n_samples: int,
        n_jobs: int,
        percent_cross: float = ...,
        percent_intra_cross: float = ...,
        percent_mutation: float = ...,
        percent_intra_mutation: float = ...,
        percent_migration: float = ...,
        percent_selection: float = ...,
        fitness_type: str = ...,
    ) -> None: ...



from __future__ import division
from numba import cuda  # type: ignore
import cupy as cp  # type: ignore
import math


class PermutationA0001:
    """PermutationA0001."""

    def __init__(self) -> None:
        """__init__.

        :rtype: None
        """
        pass

    def _permutationA0001(
        self, digits: int, repetitions: int, n_samples: int
    ) -> cp.core.core.ndarray:
        """_permutationA0001.

        :param digits:
        :type digits: int
        :param repetitions:
        :type repetitions: int
        :param n_samples:
        :type n_samples: int
        :rtype: cp.core.core.ndarray
        """

        def permutationAC0001() -> cp.core.core.ndarray:
            """permutationAC0001.

            :rtype: cp.core.core.ndarray
            """

            @cuda.jit
            def kernel(X: cp.core.core.ndarray, AL: cp.core.core.ndarray) -> None:
                row = cuda.grid(1)
                if row < X.shape[0]:
                    for j in range(0, X.shape[1], 1):
                        tmp = X[row, int(AL[row, j] * j + 0.5)]
                        X[row, int(AL[row, j] * j + 0.5)] = X[row, j]
                        X[row, j] = tmp
                cuda.syncthreads()

            X = cp.repeat(
                cp.repeat(
                    cp.expand_dims(cp.arange(digits, dtype=cp.float32), axis=0),
                    n_samples,
                    axis=0,
                ),
                repetitions,
                axis=1,
            )
            AL = cp.array(
                cp.random.rand(n_samples, digits * repetitions), dtype=cp.float32
            )

            threadsperblock = 16
            blockspergrid_x = int(math.ceil(n_samples / threadsperblock))
            blockspergrid = blockspergrid_x

            cuda.synchronize()
            kernel[blockspergrid, threadsperblock](X, AL)
            cuda.synchronize()

            AL = None

            return X

        return permutationAC0001()



import cupy as cp  # type: ignore
import numpy as np  # type: ignore
from functools import wraps
from typing import TypeVar, Callable, Any, cast, Optional, Union

TFun = TypeVar("TFun", bound=Callable[..., Any])


class SetException(Exception):
    """MyOperatorException."""

    def __init__(self, message: str) -> None:
        """__init__.

        :param message:
        :type message: str
        :rtype: None
        """
        self._message: str = self.set_message(message)

    def set_message(self, message: str) -> str:
        """set_message.

        :param message:
        :type message: str
        :rtype: str
        """
        return message

    def get_message(self) -> str:
        """get_message.

        :rtype: str
        """
        return self._message

    def __str__(self) -> str:
        """__str__.

        :rtype: str
        """
        return self.get_message()


class Check:
    def __inti__(self) -> None:
        """__inti__.

        :rtype: None
        """
        pass

    @classmethod
    def _set_n_samples_check(self, f: TFun) -> TFun:
        """_set_n_samples_check.

        :param f:
        :type f: TFun
        :rtype: TFun
        """

        @wraps(f)
        def wrapper(self, n_samples: int) -> Optional[Union[None, int]]:
            """wrapper.

            :param n_samples:
            :type n_samples: int
            :rtype: Optional[Union[None, int]]
            """
            if n_samples < 2:
                raise SetException(
                    "n_samples -> {} ; expected -> n_samples >= 2".format(
                        type(n_samples)
                    )
                )

            if type(n_samples) != int:
                raise SetException(
                    "type(n_samples) -> {} ; expected -> <class 'int'>".format(
                        type(n_samples)
                    )
                )
            return f(self, n_samples)

        return cast(TFun, wrapper)

    @classmethod
    def _set_n_jobs_check(self, f: TFun) -> TFun:
        """_set_n_jobs_check.

        :param f:
        :type f: TFun
        :rtype: TFun
        """

        @wraps(f)
        def wrapper(self, n_jobs: int) -> Optional[Union[None, int]]:
            """wrapper.

            :param n_jobs:
            :type n_jobs: int
            :rtype: Optional[Union[None, int]]
            """
            if type(n_jobs) != int:
                raise SetException(
                    "type(n_jobs) -> {} ; expected -> <class 'int'>".format(
                        type(n_jobs)
                    )
                )
            return f(self, n_jobs)

        return cast(TFun, wrapper)

    @classmethod
    def _set_n_machines_check(self, f: TFun) -> TFun:
        """_set_n_machines_check.

        :param f:
        :type f: TFun
        :rtype: TFun
        """

        @wraps(f)
        def wrapper(self, n_machines: int) -> Optional[Union[None, int]]:
            """wrapper.

            :param n_machines:
            :type n_machines: int
            :rtype: Optional[Union[None, int]]
            """
            if type(n_machines) != int:
                raise SetException(
                    "type(n_machines) -> {} ; expected -> <class 'int'>".format(
                        type(n_machines)
                    )
                )
            return f(self, n_machines)

        return cast(TFun, wrapper)

    @classmethod
    def _set_n_operations_check(self, f: TFun) -> TFun:
        """_set_n_operations_check.

        :param f:
        :type f: TFun
        :rtype: TFun
        """

        @wraps(f)
        def wrapper(self, n_operations: int) -> Optional[Union[None, int]]:
            """wrapper.

            :param n_operations:
            :type n_operations: int
            :rtype: Optional[Union[None, int]]
            """
            if type(n_operations) != int:
                raise SetException(
                    "type(n_operations) -> {} ; expected -> <class 'int'>".format(
                        type(n_operations)
                    )
                )
            return f(self, n_operations)

        return cast(TFun, wrapper)

    @classmethod
    def _set_fitness_type_check(self, f: TFun) -> TFun:
        """_set_fitness_type_check.

        :param f:
        :type f: TFun
        :rtype: TFun
        """

        @wraps(f)
        def wrapper(self, fitness_type: str) -> Optional[Union[None, str]]:
            """wrapper.

            :param fitness_type:
            :type fitness_type: str
            :rtype: Optional[Union[None, str]]
            """
            fitness_list = [
                "E_C",
                "E_L",
                "E_LT",
                "E_U",
                "E_Lw",
                "E_LTw",
                "E_Uw",
                "max_C",
            ]
            if not (fitness_type in fitness_list):
                raise SetException(
                    "fitness_type -> {} ; expected -> str in  ['E_C','E_L','E_LT','E_U','E_Lw','E_LTw','E_Uw','max_C']".format(  # noqa: E501
                        type(fitness_type)
                    )
                )
            if type(fitness_type) != str:
                raise SetException(
                    "type(fitness_type) -> {} ; expected -> <class 'str'>".format(
                        type(fitness_type)
                    )
                )
            return f(self, fitness_type)

        return cast(TFun, wrapper)

    @classmethod
    def _set_percent_cross_check(self, f: TFun) -> TFun:
        """_set_percent_cross_check.

        :param f:
        :type f: TFun
        :rtype: TFun
        """

        @wraps(f)
        def wrapper(self, percent_cross: float) -> Optional[Union[None, float]]:
            """wrapper.

            :param percent_cross:
            :type percent_cross: float
            :rtype: Optional[Union[None, float]]
            """
            if percent_cross <= 0 or percent_cross >= 1:
                raise SetException(
                    "percent_cross -> {} ; expected -> Interval[(0,1)]".format(
                        type(percent_cross)
                    )
                )

            if type(percent_cross) != float:
                raise SetException(
                    "type(percent_cross) -> {} ; expected -> <class 'float'>".format(
                        type(percent_cross)
                    )
                )
            return f(self, percent_cross)

        return cast(TFun, wrapper)

    @classmethod
    def _set_percent_intra_cross_check(self, f: TFun) -> TFun:
        """_set_percent_intra_cross_check.

        :param f:
        :type f: TFun
        :rtype: TFun
        """

        @wraps(f)
        def wrapper(self, percent_intra_cross: float) -> Optional[Union[None, float]]:
            """wrapper.

            :param percent_intra_cross:
            :type percent_intra_cross: float
            :rtype: Optional[Union[None, float]]
            """
            if percent_intra_cross <= 0 or percent_intra_cross >= 1:
                raise SetException(
                    "percent_intra_cross -> {} ; expected -> Interval[(0,1)]".format(
                        type(percent_intra_cross)
                    )
                )

            if type(percent_intra_cross) != float:
                raise SetException(
                    "type(percent_intra_cross) -> {} ; expected -> <class 'float'>".format(
                        type(percent_intra_cross)
                    )
                )
            return f(self, percent_intra_cross)

        return cast(TFun, wrapper)

    @classmethod
    def _set_percent_mutation_check(self, f: TFun) -> TFun:
        """_set_percent_mutation_check.

        :param f:
        :type f: TFun
        :rtype: TFun
        """

        @wraps(f)
        def wrapper(self, percent_mutation: float) -> Optional[Union[None, float]]:
            """wrapper.

            :param percent_mutation:
            :type percent_mutation: float
            :rtype: Optional[Union[None, float]]
            """
            if percent_mutation <= 0 or percent_mutation >= 1:
                raise SetException(
                    "percent_mutation -> {} ; expected -> Interval[(0,1)]".format(
                        type(percent_mutation)
                    )
                )
            if type(percent_mutation) != float:
                raise SetException(
                    "type(percent_mutation) -> {} ; expected -> <class 'float'>".format(
                        type(percent_mutation)
                    )
                )
            return f(self, percent_mutation)

        return cast(TFun, wrapper)

    @classmethod
    def _set_percent_intra_mutation_check(self, f: TFun) -> TFun:
        """_set_percent_intra_mutation_check.

        :param f:
        :type f: TFun
        :rtype: TFun
        """

        @wraps(f)
        def wrapper(
            self, percent_intra_mutation: float
        ) -> Optional[Union[None, float]]:
            """wrapper.

            :param percent_intra_mutation:
            :type percent_intra_mutation: float
            :rtype: Optional[Union[None, float]]
            """
            if percent_intra_mutation <= 0 or percent_intra_mutation >= 1:
                raise SetException(
                    "percent_intra_mutation -> {} ; expected -> Interval[(0,1)]".format(
                        type(percent_intra_mutation)
                    )
                )
            if type(percent_intra_mutation) != float:
                raise SetException(
                    "type(percent_intra_mutation) -> {} ; expected -> <class 'float'>".format(
                        type(percent_intra_mutation)
                    )
                )
            return f(self, percent_intra_mutation)

        return cast(TFun, wrapper)

    @classmethod
    def _set_percent_migration_check(self, f: TFun) -> TFun:
        """_set_percent_migration_check.

        :param f:
        :type f: TFun
        :rtype: TFun
        """

        @wraps(f)
        def wrapper(self, percent_migration: float) -> Optional[Union[None, float]]:
            """wrapper.

            :param percent_migration:
            :type percent_migration: float
            :rtype: Optional[Union[None, float]]
            """
            if percent_migration <= 0 or percent_migration >= 1:
                raise SetException(
                    "percent_migration -> {} ; expected -> Interval[(0,1)]".format(
                        type(percent_migration)
                    )
                )
            if type(percent_migration) != float:
                raise SetException(
                    "type(percent_migration) -> {} ; expected -> <class 'float'>".format(
                        type(percent_migration)
                    )
                )
            return f(self, percent_migration)

        return cast(TFun, wrapper)

    @classmethod
    def _set_percent_selection_check(self, f: TFun) -> TFun:
        """_set_percent_selection_check.

        :param f:
        :type f: TFun
        :rtype: TFun
        """

        @wraps(f)
        def wrapper(self, percent_selection: float) -> Optional[Union[None, float]]:
            """wrapper.

            :param percent_selection:
            :type percent_selection: float
            :rtype: Optional[Union[None, float]]
            """
            if percent_selection <= 0 or percent_selection >= 1:
                raise SetException(
                    "percent_selection -> {} ; expected -> Interval[(0,1)]".format(
                        type(percent_selection)
                    )
                )
            if type(percent_selection) != float:
                raise SetException(
                    "type(percent_selection) -> {} ; expected -> <class 'float'>".format(
                        type(percent_selection)
                    )
                )
            return f(self, percent_selection)

        return cast(TFun, wrapper)

    @classmethod
    def _set_fitness_check(self, f: TFun) -> TFun:
        """_set_fitness_check.

        :param f:
        :type f: TFun
        :rtype: TFun
        """

        @wraps(f)
        def wrapper(
            self,
            fitness: cp.core.core.ndarray,
        ) -> Optional[Union[None, cp.core.core.ndarray]]:
            """wrapper.

            :param fitness:
            :type fitness: cp.core.core.ndarray
            :rtype: Optional[Union[None, cp.core.core.ndarray]]
            """
            if type(fitness) != cp.core.core.ndarray:
                raise SetException(
                    "type(fitness) -> {} ; expected -> <class 'cp.core.core.ndarray'>".format(
                        type(fitness)
                    )
                )
            return f(self, fitness)

        return cast(TFun, wrapper)

    @classmethod
    def _set_population_check(self, f: TFun) -> TFun:
        """_set_population_check.

        :param f:
        :type f: TFun
        :rtype: TFun
        """

        @wraps(f)
        def wrapper(
            self, population: Optional[Union[cp.core.core.ndarray, None]] = None
        ) -> Optional[Union[None, cp.core.core.ndarray]]:
            """wrapper.

            :param population:
            :type population: Optional[Union[cp.core.core.ndarray, None]]
            :rtype: Optional[Union[None, cp.core.core.ndarray]]
            """
            if type(population) != cp.core.core.ndarray:
                if not (population is None):
                    raise SetException(
                        "type(population) -> {} ; expected -> <class 'cp.core.core.ndarray'>".format(
                            type(population)
                        )
                    )
            return f(self, population)

        return cast(TFun, wrapper)

    @classmethod
    def _set_processing_time_check(self, f: TFun) -> TFun:
        """_set_processing_time_check.

        :param f:
        :type f: TFun
        :rtype: TFun
        """

        @wraps(f)
        def wrapper(
            self,
            processing_time: Optional[Union[list, np.ndarray, cp.core.core.ndarray]],
        ) -> cp.core.core.ndarray:
            """wrapper.

            :param processing_time:
            :type processing_time: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
            :rtype: cp.core.core.ndarray
            """
            if not (
                type(processing_time) == list
                or type(processing_time) == np.ndarray
                or type(processing_time) == cp.core.core.ndarray
            ):
                raise SetException(
                    "type(processing_time) -> {} ; expected -> <class 'Optional[Union[list, np.ndarray, cp.core.core.ndarray]]'>".format(  # noqa: E501
                        type(processing_time)
                    )
                )
            return f(self, processing_time)

        return cast(TFun, wrapper)

    @classmethod
    def _set_machine_sequence_check(self, f: TFun) -> TFun:
        """_set_machine_sequence_check.

        :param f:
        :type f: TFun
        :rtype: TFun
        """

        @wraps(f)
        def wrapper(
            self,
            machine_sequence: Optional[Union[list, np.ndarray, cp.core.core.ndarray]],
        ) -> cp.core.core.ndarray:
            """wrapper.

            :param machine_sequence:
            :type machine_sequence: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
            :rtype: cp.core.core.ndarray
            """
            if not (
                type(machine_sequence) == list
                or type(machine_sequence) == np.ndarray
                or type(machine_sequence) == cp.core.core.ndarray
            ):
                raise SetException(
                    "type(machine_sequence) -> {} ; expected -> <class 'Optional[Union[list, np.ndarray, cp.core.core.ndarray]]'>".format(  # noqa: E501
                        type(machine_sequence)
                    )
                )
            return f(self, machine_sequence)

        return cast(TFun, wrapper)

    @classmethod
    def _set_due_date_check(self, f: TFun) -> TFun:
        """_set_due_date_check.

        :param f:
        :type f: TFun
        :rtype: TFun
        """

        @wraps(f)
        def wrapper(
            self, due_date: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
        ) -> cp.core.core.ndarray:
            """wrapper.

            :param due_date:
            :type due_date: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
            :rtype: cp.core.core.ndarray
            """
            if not (
                type(due_date) == list
                or type(due_date) == np.ndarray
                or type(due_date) == cp.core.core.ndarray
            ):
                raise SetException(
                    "type(due_date) -> {} ; expected -> <class 'Optional[Union[list, np.ndarray, cp.core.core.ndarray]]'>".format(  # noqa: E501
                        type(due_date)
                    )
                )
            return f(self, due_date)

        return cast(TFun, wrapper)

    @classmethod
    def _set_weights_check(self, f: TFun) -> TFun:
        """_set_weights_check.

        :param f:
        :type f: TFun
        :rtype: TFun
        """

        @wraps(f)
        def wrapper(
            self, weights: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
        ) -> cp.core.core.ndarray:
            """wrapper.

            :param weights:
            :type weights: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
            :rtype: cp.core.core.ndarray
            """
            if not (
                type(weights) == list
                or type(weights) == np.ndarray
                or type(weights) == cp.core.core.ndarray
            ):
                raise SetException(
                    "type(weights) -> {} ; expected -> <class 'Optional[Union[list, np.ndarray, cp.core.core.ndarray]]'>".format(  # noqa: E501
                        type(weights)
                    )
                )
            return f(self, weights)

        return cast(TFun, wrapper)

    @classmethod
    def _set_percents_c_m_m_s_check(self, f: TFun) -> TFun:
        """_set_percent_mutation_check.

        :param f:
        :type f: TFun
        :rtype: TFun
        """

        @wraps(f)
        def wrapper(
            self,
            percent_cross: float,
            percent_mutation: float,
            percent_migration: float,
            percent_selection: float,
        ) -> None:
            """wrapper.

            :param percent_cross:
            :type percent_cross: float
            :param percent_mutation:
            :type percent_mutation: float
            :param percent_migration:
            :type percent_migration: float
            :param percent_selection:
            :type percent_selection: float
            :rtype: None
            """
            if (
                percent_cross > 1 - percent_selection
                or percent_mutation > 1 - percent_selection
                or percent_migration > 1 - percent_selection
            ):
                raise SetException(
                    "False ; expected ->  True  where percent_cross <= 1 - percent_selection and percent_mutation <= 1 - percent_selection and percent_migration <= 1 - percent_selection"  # noqa: E501
                )
            return f(
                self,
                percent_cross,
                percent_mutation,
                percent_migration,
                percent_selection,
            )

        return cast(TFun, wrapper)



from __future__ import division
from typing import Optional, Union
import numpy as np  # type: ignore
import cupy as cp  # type: ignore


from gepapy.exceptions.operations import Check
from gepapy.kernels.permutationA0001 import PermutationA0001
from gepapy.kernels.crossA0001 import CrossA0001
from gepapy.kernels.mutationA0001 import MutationA0001
from gepapy.kernels.sortA0001 import SortA0001
from gepapy.kernels.fitnessA0001 import FitnessA0001


class Operations(PermutationA0001, CrossA0001, MutationA0001, SortA0001, FitnessA0001):
    """Operations."""

    def __init__(self) -> None:
        """__init__.

        :rtype: None
        """
        self._initialized: bool
        self._n_samples: int
        self._n_jobs: int
        self._n_machines: int
        self._n_operations: int
        self._fitness_type: cp.core.core.ndarray
        self._percent_cross: float
        self._percent_intra_cross: float
        self._percent_mutation: float
        self._percent_intra_mutation: float
        self._percent_migration: float
        self._percent_selection: float
        self._fitness: cp.core.core.ndarray
        self._population: cp.core.core.ndarray
        self._processing_time: cp.core.core.ndarray
        self._machine_sequence: cp.core.core.ndarray
        self._due_date: cp.core.core.ndarray
        self._weights: cp.core.core.ndarray

    @Check._set_n_samples_check
    def set_n_samples(self, n_samples: int) -> Optional[Union[None, int]]:
        """set_n_samples.

        :param n_samples:
        :type n_samples: int
        :rtype: int
        """
        if self._initialized:
            self._n_samples = n_samples
            return None
        else:
            return n_samples

    @Check._set_n_jobs_check
    def _set_n_jobs(self, n_jobs: int) -> Optional[Union[None, int]]:
        """_set_n_jobs.

        :param n_jobs:
        :type n_jobs: int
        :rtype: int
        """
        if self._initialized:
            self._n_jobs = n_jobs
            return None
        else:
            return n_jobs

    @Check._set_n_machines_check
    def _set_n_machines(self, n_machines: int) -> Optional[Union[None, int]]:
        """_set_n_machines.

        :param n_machines:
        :type n_machines: int
        :rtype: int
        """
        if self._initialized:
            self._n_machines = n_machines
            return None
        else:
            return n_machines

    @Check._set_n_operations_check
    def _set_n_operations(self, n_operations: int) -> Optional[Union[None, int]]:
        """_set_n_operations.

        :param n_operations:
        :type n_operations: int
        :rtype: int
        """
        if self._initialized:
            self._n_operations = n_operations
            return None
        else:
            return n_operations

    @Check._set_fitness_type_check
    def set_fitness_type(self, fitness_type: str) -> Optional[Union[None, str]]:
        """_set_fitness_type.

        :param fitness_type:
        :type fitness_type: str
        :rtype: str
        """
        if self._initialized:
            self._fitness_type = fitness_type
            return None
        else:
            return fitness_type

    @Check._set_percent_cross_check
    def _set_percent_cross(self, percent_cross: float) -> Optional[Union[None, float]]:
        """set_percent_cross.

        :param percent_cross:
        :type percent_cross: float
        :rtype: float
        """
        if self._initialized:
            self._percent_cross = percent_cross
            return None
        else:
            return percent_cross

    @Check._set_percent_intra_cross_check
    def set_percent_intra_cross(
        self, percent_intra_cross: float
    ) -> Optional[Union[None, float]]:
        """set_percent_intra_cross.

        :param percent_intra_cros:
        :type percent_intra_cros: float
        :rtype: float
        """
        if self._initialized:
            self._percent_intra_cross = percent_intra_cross
            return None
        else:
            return percent_intra_cross

    @Check._set_percent_mutation_check
    def _set_percent_mutation(
        self, percent_mutation: float
    ) -> Optional[Union[None, float]]:
        """set_percent_mutation.

        :param percent_mutation:
        :type percent_mutation: float
        :rtype: float
        """
        if self._initialized:
            self._percent_mutation = percent_mutation
            return None
        else:
            return percent_mutation

    @Check._set_percent_intra_mutation_check
    def set_percent_intra_mutation(
        self, percent_intra_mutation: float
    ) -> Optional[Union[None, float]]:
        """set_percent_intra_mutation.

        :param percent_intra_mutation:
        :type percent_intra_mutation: float
        :rtype: float
        """
        if self._initialized:
            self._percent_intra_mutation = percent_intra_mutation
            return None
        else:
            return percent_intra_mutation

    @Check._set_percent_migration_check
    def _set_percent_migration(
        self, percent_selection: float
    ) -> Optional[Union[None, float]]:
        """set_percent_migration.

        :param percent_selection:
        :type percent_selection: float
        :rtype: float
        """
        if self._initialized:
            self._percent_selection = percent_selection
            return None
        else:
            return percent_selection

    @Check._set_percent_selection_check
    def _set_percent_selection(
        self, percent_migration: float
    ) -> Optional[Union[None, float]]:
        """set_percent_selection.

        :param percent_migration:
        :type percent_migration: float
        :rtype: float
        """
        if self._initialized:
            self._percent_migration = percent_migration
            return None
        else:
            return percent_migration

    @Check._set_fitness_check
    def _set_fitness(
        self, fitness: cp.core.core.ndarray
    ) -> Optional[Union[None, cp.core.core.ndarray]]:
        """_set_fitness.

        :param fitness:
        :type fitness: cp.core.core.ndarray
        :rtype: None
        """
        if self._initialized:
            self._fitness = fitness
            return None
        else:
            return fitness

    @Check._set_population_check
    def set_population(
        self, population: Optional[Union[cp.core.core.ndarray, None]] = None
    ) -> Optional[Union[None, cp.core.core.ndarray]]:
        """_set_population.

        :param population:
        :type population: Optional[Union[cp.core.core.ndarray, None]]
        """
        if self._initialized:
            self._population = population
            return None
        else:
            return self.exec_permutationA0001()

    @Check._set_processing_time_check
    def _set_processing_time(
        self, processing_time: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
    ) -> cp.core.core.ndarray:
        """_set_processing_time.

        :param processing_time:
        :type processing_time: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
        :rtype: cp.core.core.ndarray
        """
        return cp.array(processing_time, dtype=cp.float32)

    @Check._set_machine_sequence_check
    def _set_machine_sequence(
        self, machine_sequence: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
    ) -> cp.core.core.ndarray:
        """_set_machine_sequence.

        :param machine_sequence:
        :type machine_sequence: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
        :rtype: cp.core.core.ndarray
        """
        return cp.array(machine_sequence, dtype=cp.float32)

    @Check._set_due_date_check
    def _set_due_date(
        self, due_date: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
    ) -> cp.core.core.ndarray:
        """_set_due_date.

        :param due_date:
        :type due_date: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
        :rtype: cp.core.core.ndarray
        """
        due_date = cp.array(due_date, dtype=cp.float32)
        return due_date

    @Check._set_weights_check
    def _set_weights(
        self, weights: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
    ) -> cp.core.core.ndarray:
        """_set_weights.

        :param weights:
        :type weights: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
        :rtype: cp.core.core.ndarray
        """
        weights = cp.array(weights, dtype=cp.float32)
        return weights

    @Check._set_percents_c_m_m_s_check
    def set_percents_c_m_m_s(
        self,
        percent_cross: float,
        percent_mutation: float,
        percent_migration: float,
        percent_selection: float,
    ) -> None:
        """set_percents_c_m_m_s.

        :param percent_cross:
        :type percent_cross: float
        :param percent_mutation:
        :type percent_mutation: float
        :param percent_migration:
        :type percent_migration: float
        :param percent_selection:
        :type percent_selection: float
        :rtype: None
        :constraint: percent_cross and percent_mutation and percent_migration <=  1 - percent_selection
        """
        self._set_percent_cross(percent_cross)
        self._set_percent_mutation(percent_mutation)
        self._set_percent_migration(percent_migration)
        self._set_percent_selection(percent_selection)

    def get_n_samples(self) -> int:
        """get_n_samples.

        :rtype: int
        """
        return self._n_samples

    def get_n_jobs(self) -> int:
        """get_n_jobs.

        :rtype: int
        """
        return self._n_jobs

    def get_n_machines(self) -> int:
        """get_n_machines.

        :rtype: int
        """
        return self._n_machines

    def get_n_operations(self) -> int:
        """get_n_operations.

        :rtype: int
        """
        return self._n_operations

    def get_fitness_type(self) -> cp.core.core.ndarray:
        """get_fitness_type.

        :rtype: cp.core.core.ndarray
        """
        return self._fitness_type

    def get_percent_cross(self) -> float:
        """get_percent_cross.

        :rtype: float
        """
        return self._percent_cross

    def get_percent_intra_cross(self) -> float:
        """get_percent_intra_cross.

        :rtype: float
        """
        return self._percent_intra_cross

    def get_percent_mutation(self) -> float:
        """get_percent_mutation.

        :rtype: float
        """
        return self._percent_mutation

    def get_percent_intra_mutation(self) -> float:
        """get_percent_intra_mutation.

        :rtype: float
        """
        return self._percent_intra_mutation

    def get_percent_migration(self) -> float:
        """get_percent_migration.

        :rtype: float
        """
        return self._percent_migration

    def get_percent_selection(self) -> float:
        """get_percent_selection.

        :rtype: float
        """
        return self._percent_selection

    def get_fitness(self) -> cp.core.core.ndarray:
        """get_fitness.

        :rtype: cp.core.core.ndarray
        """
        return self._fitness

    def get_population(self) -> cp.core.core.ndarray:
        """get_population.

        :rtype: cp.core.core.ndarray
        """
        return self._population

    def get_processing_time(self) -> cp.core.core.ndarray:
        """get_processing_time.

        :rtype: cp.core.core.ndarray
        """
        return self._processing_time

    def get_machine_sequence(self) -> cp.core.core.ndarray:
        """get_machine_sequence.

        :rtype: cp.core.core.ndarray
        """
        return self._machine_sequence

    def get_due_date(self) -> cp.core.core.ndarray:
        """get_due_date.

        :rtype: cp.core.core.ndarray
        """
        return self._due_date

    def get_weights(self) -> cp.core.core.ndarray:
        """get_weights.

        :rtype: cp.core.core.ndarray
        """
        return self._weights

    def exec_permutationA0001(self) -> cp.core.core.ndarray:
        """exec_permutationA0001.

        :rtype: cp.core.core.ndarray
        """
        return self._permutationA0001(
            self.get_n_jobs(), self.get_n_operations(), self.get_n_samples()
        )

    def exec_crossA0001(self) -> None:
        """exec_crossA0001.

        :rtype: None
        """
        x_population = cp.copy(self.get_population())
        x_aux = cp.copy(self.get_population())
        index_selection = int(self.get_n_samples() * self.get_percent_selection())
        index_cross = int(self.get_n_samples() * self.get_percent_cross())
        cp.random.shuffle(x_population)
        if index_cross % 2 == 0:
            y_population = self._crossA0001(
                x_population[index_selection:, :][0:index_cross, :],
                self.get_n_jobs(),
                self.get_n_operations(),
                index_cross,
                self.get_percent_intra_cross(),
            )
            x_aux[index_selection:, :][-index_cross:, :] = y_population
            self.set_population(x_aux)
        else:
            index_cross -= 1
            y_population = self._crossA0001(
                x_population[index_selection:, :][0:index_cross, :],
                self.get_n_jobs(),
                self.get_n_operations(),
                self.get_n_samples(),
                self.get_percent_intra_cross(),
            )
            x_aux[index_selection:, :][-index_cross:, :] = y_population
            self.set_population(x_aux)

    def exec_mutationA0001(self) -> None:
        """exec_mutationA0001.

        :rtype: None
        """
        x_population = cp.copy(self.get_population())
        x_aux = cp.copy(self.get_population())
        index_selection = int(self.get_n_samples() * self.get_percent_selection())
        index_mutation = int(self.get_n_samples() * self.get_percent_mutation())
        cp.random.shuffle(x_population)
        y_population = self._mutationA0001(
            x_population[index_selection:, :][-index_mutation:, :],
            self.get_n_jobs(),
            self.get_n_operations(),
            index_mutation,
            self.get_percent_intra_mutation(),
        )
        x_aux[index_selection:, :][0:index_mutation, :] = y_population
        self.set_population(x_aux)

    def exec_migrationA0001(self) -> None:
        """exec_migrationA0001.

        :rtype: None
        """
        x_population = cp.copy(self.get_population())
        x_aux = cp.copy(self.get_population())
        index_selection = int(self.get_n_samples() * self.get_percent_selection())
        index_migration = int(self.get_n_samples() * self.get_percent_migration())
        cp.random.shuffle(x_population)
        y_population = self._permutationA0001(
            self.get_n_jobs(), self.get_n_operations(), index_migration
        )
        x_aux[index_selection:, :][-index_migration:, :] = y_population
        self.set_population(x_aux)

    def exec_sortA0001(self) -> None:
        """exec_sortA0001.

        :rtype: None
        """
        x_population = cp.copy(self.get_population())
        x_sort = cp.copy(self.get_fitness())
        y_population, y_sort = self._sortA0001(
            x_population,
            x_sort,
            self.get_n_jobs(),
            self.get_n_operations(),
            self.get_n_samples(),
        )
        self.set_population(y_population)
        self._set_fitness(y_sort)

    def exec_fitnessA0001(self) -> None:
        """exec_fitnessA0001.

        :rtype: None
        """
        x_population = cp.copy(self.get_population())
        fitness = self._fitnessA0001(
            x_population,
            self.get_due_date(),
            self.get_weights(),
            self.get_processing_time(),
            self.get_machine_sequence(),
            self.get_n_jobs(),
            self.get_n_samples(),
            self.get_n_machines(),
        )
        self._set_fitness(fitness[self.get_fitness_type()])

    def get_plan(self, row: int, fact_conv: int, start_time: int) -> list:
        """get_plan.

        :param row:
        :type row: int
        :param fact_conv:
        :type fact_conv: int
        :param start_time:
        :type start_time: int
        :rtype: dict
        """
        x_population = cp.copy(self.get_population())
        return self._get_planA0001(
            row,
            x_population,
            self.get_processing_time(),
            self.get_machine_sequence(),
            self.get_n_jobs(),
            self.get_n_machines(),
            fact_conv,
            start_time,
        )



import cupy as cp  # type: ignore
import numpy as np  # type: ignore
from typing import Optional as Optional
from typing import Union as Union
from gepapy.kernels.crossA0001 import CrossA0001 as CrossA0001
from gepapy.kernels.fitnessA0001 import FitnessA0001 as FitnessA0001
from gepapy.kernels.mutationA0001 import MutationA0001 as MutationA0001
from gepapy.kernels.permutationA0001 import (
    PermutationA0001 as PermutationA0001,
)
from gepapy.kernels.sortA0001 import SortA0001 as SortA0001

class Operations(PermutationA0001, CrossA0001, MutationA0001, SortA0001, FitnessA0001):
    def __init__(self) -> None: ...
    def set_n_samples(self, n_samples: int) -> int: ...
    def _set_n_jobs(self, n_jobs: int) -> Optional[Union[None, int]]: ...
    def _set_n_machines(self, n_machines: int) -> Optional[Union[None, int]]: ...
    def _set_n_operations(self, n_operations: int) -> Optional[Union[None, int]]: ...
    def set_fitness_type(self, fitness_type: str) -> Optional[Union[None, str]]: ...
    def _set_percent_cross(self, percent_cross: float) -> float: ...
    def set_percent_intra_cross(self, percent_intra_cros: float) -> float: ...
    def _set_percent_mutation(self, percent_mutation: float) -> float: ...
    def set_percent_intra_mutation(self, percent_intra_mutation: float) -> float: ...
    def _set_percent_migration(self, percent_selection: float) -> float: ...
    def _set_percent_selection(self, percent_migration: float) -> float: ...
    def _set_fitness(
        self, fitness: cp.core.core.ndarray
    ) -> Optional[Union[None, cp.core.core.ndarray]]: ...
    def set_population(
        self, population: Optional[Union[cp.core.core.ndarray, None]] = None
    ) -> Optional[Union[None, cp.core.core.ndarray]]: ...
    def _set_processing_time(
        self, processing_time: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
    ) -> cp.core.core.ndarray: ...
    def _set_machine_sequence(
        self, machine_sequence: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
    ) -> cp.core.core.ndarray: ...
    def _set_due_date(
        self, due_date: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
    ) -> cp.core.core.ndarray: ...
    def _set_weights(
        self, weights: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
    ) -> cp.core.core.ndarray: ...
    def get_n_samples(self) -> int: ...
    def get_n_jobs(self) -> int: ...
    def get_n_machines(self) -> int: ...
    def get_n_operations(self) -> int: ...
    def get_fitness_type(self) -> cp.core.core.ndarray: ...
    def get_percent_cross(self) -> float: ...
    def get_percent_intra_cross(self) -> float: ...
    def get_percent_mutation(self) -> float: ...
    def get_percent_intra_mutation(self) -> float: ...
    def get_percent_migration(self) -> float: ...
    def get_percent_selection(self) -> float: ...
    def get_fitness(self) -> cp.core.core.ndarray: ...
    def get_population(self) -> cp.core.core.ndarray: ...
    def get_processing_time(self) -> cp.core.core.ndarray: ...
    def get_machine_sequence(self) -> cp.core.core.ndarray: ...
    def get_due_date(self) -> cp.core.core.ndarray: ...
    def get_weights(self) -> cp.core.core.ndarray: ...
    def exec_permutationA0001(self) -> cp.core.core.ndarray: ...
    def exec_crossA0001(self) -> None: ...
    def exec_mutationA0001(self) -> None: ...
    def exec_migrationA0001(self) -> None: ...
    def exec_sortA0001(self) -> None: ...
    def exec_fitnessA0001(self) -> None: ...
    def get_plan(self, row: int, fact_conv: int, start_time: int) -> list: ...



from __future__ import division
from typing import Union, Optional
import numpy as np  # type: ignore
import cupy as cp  # type: ignore

from gepapy.operations import Operations


class Job_Shop(Operations):
    """Job_Shop."""

    def __init__(
        self,
        processing_time: Optional[Union[list, np.ndarray, cp.core.core.ndarray]],
        machine_sequence: Optional[Union[list, np.ndarray, cp.core.core.ndarray]],
        due_date: Optional[Union[list, np.ndarray, cp.core.core.ndarray]],
        weights: Optional[Union[list, np.ndarray, cp.core.core.ndarray]],
        n_samples: int,
        n_jobs: int,
        n_operations: int,
        n_machines: int,
        percent_cross: float = 0.2,
        percent_intra_cross: float = 0.5,
        percent_mutation: float = 0.2,
        percent_intra_mutation: float = 0.5,
        percent_migration: float = 0.1,
        percent_selection: float = 0.1,
        fitness_type: str = "max_C",
    ):
        """__init__.

        :param processing_time:
        :type processing_time: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
        :param machine_sequence:
        :type machine_sequence: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
        :param due_date:
        :type due_date: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
        :param weights:
        :type weights: Optional[Union[list, np.ndarray, cp.core.core.ndarray]]
        :param n_samples:
        :type n_samples: int
        :param n_jobs:
        :type n_jobs: int
        :param n_operations:
        :type n_operations: int
        :param n_machines:
        :type n_machines: int
        :param percent_cross:
        :type percent_cross: float
        :param percent_intra_cross:
        :type percent_intra_cross: float
        :param percent_mutation:
        :type percent_mutation: float
        :param percent_intra_mutation:
        :type percent_intra_mutation: float
        :param percent_migration:
        :type percent_migration: float
        :param percent_selection:
        :type percent_selection: float
        :param fitness_type:
        :type fitness_type: str
        """
        self._initialized = False
        self._n_samples = self.set_n_samples(n_samples)
        self._n_jobs = self._set_n_jobs(n_jobs)
        self._n_machines = self._set_n_machines(n_machines)
        self._n_operations = self._set_n_operations(n_operations)
        self._fitness_type = self.set_fitness_type(fitness_type)
        self._processing_time = self._set_processing_time(processing_time)
        self._machine_sequence = self._set_machine_sequence(machine_sequence)
        self._due_date = self._set_due_date(due_date)
        self._weights = self._set_weights(weights)
        self._percent_cross = self._set_percent_cross(percent_cross)
        self._percent_intra_cross = self.set_percent_intra_cross(percent_intra_cross)
        self._percent_mutation = self._set_percent_mutation(percent_mutation)
        self._percent_intra_mutation = self.set_percent_intra_mutation(
            percent_intra_mutation
        )
        self._percent_migration = self._set_percent_migration(percent_migration)
        self._percent_selection = self._set_percent_selection(percent_selection)
        self._fitness = cp.array([], dtype=cp.float32)
        self._population = cp.array([], dtype=cp.float32)
        self._population = self.set_population()
        self._initialized = True



from __future__ import division
from numba import cuda  # type: ignore
import numpy as np  # type: ignore
import cupy as cp  # type: ignore
import math


class FitnessA0001:
    """FitnessA0001."""

    def __init__(self) -> None:
        """__init__.

        :rtype: None
        """
        pass

    def _fitnessA0001(
        self,
        x: cp.core.core.ndarray,
        d: cp.core.core.ndarray,
        w: cp.core.core.ndarray,
        T: cp.core.core.ndarray,
        M: cp.core.core.ndarray,
        digits: int,
        n_samples: int,
        n_machines: int,
    ) -> cp.core.core.ndarray:
        """_fitnessA0001.

        :param x:
        :type x: cp.core.core.ndarray
        :param d:
        :type d: cp.core.core.ndarray
        :param w:
        :type w: cp.core.core.ndarray
        :param T:
        :type T: cp.core.core.ndarray
        :param M:
        :type M: cp.core.core.ndarray
        :param digits:
        :type digits: int
        :param n_samples:
        :type n_samples: int
        :param n_machines:
        :type n_machines: int
        :rtype: cp.core.core.ndarray
        """

        def fitnessAC0001():
            """fitnessAC0001."""

            @cuda.jit
            def kernel(x, T, M, digits, n_samples, n_machines, c_o, t_j, t_m):
                row = cuda.grid(1)
                if row < n_samples:
                    for i in range(x.shape[1]):
                        idx = int(math.ceil(x[row, i]))
                        t_aux = int(
                            math.ceil(T[row, idx, int(math.ceil(c_o[row, idx]))])
                        )
                        m_aux = int(
                            math.ceil(M[row, idx, int(math.ceil(c_o[row, idx]))])
                        )
                        c_o[row, idx] = c_o[row, idx] + 1

                        if t_m[row, m_aux] > t_j[row, idx]:
                            t_m[row, m_aux] = t_m[row, m_aux] + t_aux
                            t_j[row, idx] = t_m[row, m_aux]

                        else:
                            t_j[row, idx] = t_j[row, idx] + t_aux
                            t_m[row, m_aux] = t_j[row, idx]

            T_expand = cp.array(
                cp.repeat(cp.expand_dims(T, axis=0), n_samples, axis=0),
                dtype=cp.float32,
            )
            M_expand = cp.array(
                cp.repeat(cp.expand_dims(M, axis=0), n_samples, axis=0),
                dtype=cp.float32,
            )
            c_o_expand = cp.array(cp.zeros([n_samples, digits]), dtype=cp.float32)
            t_j_expand = cp.array(cp.zeros([n_samples, digits]), dtype=cp.float32)
            t_m_expand = cp.array(cp.zeros([n_samples, n_machines]), dtype=cp.float32)
            threadsperblock = 16
            blockspergrid_x = int(math.ceil(n_samples / threadsperblock))
            blockspergrid = blockspergrid_x

            cuda.synchronize()
            kernel[blockspergrid, threadsperblock](
                x,
                T_expand,
                M_expand,
                digits,
                n_samples,
                n_machines,
                c_o_expand,
                t_j_expand,
                t_m_expand,
            )
            cuda.synchronize()

            return t_j_expand

        C = fitnessAC0001()

        def fitnessAC0002() -> dict:
            """fitnessAC0002.

            :rtype: dict
            """
            d_expand = cp.array(
                cp.repeat(cp.expand_dims(d, axis=0), n_samples, axis=0),
                dtype=cp.float32,
            )
            w_expand = cp.array(
                cp.repeat(cp.expand_dims(w, axis=0), n_samples, axis=0),
                dtype=cp.float32,
            )
            L = C - d_expand
            LT = cp.where(L > 0, L, 0)
            U = cp.where(L > 0, L, 0)
            Lw = L * w_expand
            LTw = LT * w_expand
            Uw = U * w_expand
            E_C = cp.sum(C, axis=1)
            E_L = cp.sum(L, axis=1)
            E_LT = cp.sum(LT, axis=1)
            E_U = cp.sum(U, axis=1)
            E_Lw = cp.sum(Lw, axis=1)
            E_LTw = cp.sum(LTw, axis=1)
            E_Uw = cp.sum(Uw, axis=1)
            max_C = cp.max(C, axis=1)

            return {
                "E_C": E_C,
                "E_L": E_L,
                "E_LT": E_LT,
                "E_U": E_U,
                "E_Lw": E_Lw,
                "E_LTw": E_LTw,
                "E_Uw": E_Uw,
                "max_C": max_C,
            }

        return fitnessAC0002()

    def _get_planA0001(
        self,
        row: int,
        X: cp.core.core.ndarray,
        T: cp.core.core.ndarray,
        M: cp.core.core.ndarray,
        digits: int,
        n_machines: int,
        fact_conv: float,
        start_time: int,
    ) -> list:
        """_get_planA0001.

        :param row:
        :type row: int
        :param X:
        :type X: cp.core.core.ndarray
        :param T:
        :type T: cp.core.core.ndarray
        :param M:
        :type M: cp.core.core.ndarray
        :param digits:
        :type digits: int
        :param n_machines:
        :type n_machines: int
        :param fact_conv:
        :type fact_conv: float
        :param start_time:
        :type start_time: int
        :rtype: list
        """

        x = cp.asnumpy(X[row].copy())
        T = cp.asnumpy(T)
        M = cp.asnumpy(M)
        c_o = np.zeros(digits, dtype=int)
        t_m = np.zeros(n_machines, dtype=int)
        t_j = np.zeros(digits, dtype=int)

        plan = []

        for i in range(x.shape[0]):
            idx = int(math.ceil(x[i]))
            t_aux = int(math.ceil(T[idx, int(math.ceil(c_o[idx]))]))
            m_aux = int(math.ceil(M[idx, int(math.ceil(c_o[idx]))]))
            c_o[idx] = c_o[idx] + 1

            if t_m[m_aux] > t_j[idx]:
                plan.append(
                    {
                        "Finish": (t_m[m_aux] + t_aux) * fact_conv + start_time,
                        "Resource": "Job {}".format(idx),
                        "Start": (t_m[m_aux]) * fact_conv + start_time,
                        "Task": "Machine {}".format(m_aux),
                    }
                )
                t_m[m_aux] = t_m[m_aux] + t_aux
                t_j[idx] = t_m[m_aux]

            else:
                plan.append(
                    {
                        "Finish": (t_j[idx] + t_aux) * fact_conv + start_time,
                        "Resource": "Job {}".format(idx),
                        "Start": (t_j[idx]) * fact_conv + start_time,
                        "Task": "Machine {}".format(m_aux),
                    }
                )
                t_j[idx] = t_j[idx] + t_aux
                t_m[m_aux] = t_j[idx]
        return plan



import cupy as cp  # type: ignore

class CrossA0001:
    def __init__(self) -> None: ...
    def _crossA0001(
        self,
        X: cp.core.core.ndarray,
        digits: int,
        repetitions: int,
        n_samples: int,
        percent_c: float,
    ) -> cp.core.core.ndarray: ...



from __future__ import division
import cupy as cp  # type: ignore


class SortA0001:
    """SortA0001."""

    def __init__(self) -> None:
        """__init__.

        :rtype: None
        """
        pass

    def _sortA0001(
        self,
        X: cp.core.core.ndarray,
        y: cp.core.core.ndarray,
        digits: int,
        repetitions: int,
        n_samples: int,
    ) -> cp.core.core.ndarray:
        """_sortA0001.

        :param X:
        :type X: cp.core.core.ndarray
        :param y:
        :type y: cp.core.core.ndarray
        :param digits:
        :type digits: int
        :param repetitions:
        :type repetitions: int
        :param n_samples:
        :type n_samples: int
        :rtype: cp.core.core.ndarray
        """

        def sortAC0001() -> cp.core.core.ndarray:
            """sortAC0001.

            :rtype: cp.core.core.ndarray
            """

            X_AUX = cp.copy(X)
            return X_AUX[y.argsort()], cp.sort(y)

        return sortAC0001()



import cupy as cp  # type: ignore

class PermutationA0001:
    def __init__(self) -> None: ...
    def _permutationA0001(
        self, digits: int, repetitions: int, n_samples: int
    ) -> cp.core.core.ndarray: ...



from __future__ import division
from numba import cuda  # type: ignore
import cupy as cp  # type: ignore
import math


class CrossA0001:
    """CrossA0001."""

    def __init__(self) -> None:
        """__init__.

        :rtype: None
        """
        pass

    def _crossA0001(
        self,
        X: cp.core.core.ndarray,
        digits: int,
        repetitions: int,
        n_samples: int,
        percent_c: float,
    ) -> cp.core.core.ndarray:
        """_crossA0001.

        :param X:
        :type X: cp.core.core.ndarray
        :param digits:
        :type digits: int
        :param repetitions:
        :type repetitions: int
        :param n_samples:
        :type n_samples: int
        :param percent_c:
        :type percent_c: float
        :rtype: cp.core.core.ndarray
        """

        def crossAC0001() -> cp.core.core.ndarray:
            """crossAC0001.

            :rtype: cp.core.core.ndarray
            """

            @cuda.jit
            def kernel(
                X: cp.core.core.ndarray,
                X_AUX: cp.core.core.ndarray,
                x_dim_1: int,
                percent_c: float,
            ) -> None:
                """kernel.

                :param X:
                :type X: cp.core.core.ndarray
                :param X_AUX:
                :type X_AUX: cp.core.core.ndarray
                :param x_dim_1:
                :type x_dim_1: int
                :param percent_c:
                :type percent_c: float
                :rtype: None
                """
                row = cuda.grid(1)
                if row < n_samples:
                    if row % 2 == 0:
                        for i in range(0, int(math.ceil(x_dim_1 * percent_c)), 1):
                            X_AUX[row, i] = X[row, i]

                    else:
                        for i in range(int(math.ceil(x_dim_1 * percent_c)), x_dim_1, 1):
                            X_AUX[row, i] = X[row, i]

            x_dim_1 = digits * repetitions
            X_AUX = cp.zeros([n_samples, x_dim_1])

            threadsperblock = 16
            blockspergrid_x = int(math.ceil((n_samples) / threadsperblock))
            blockspergrid = blockspergrid_x

            cuda.synchronize()
            kernel[blockspergrid, threadsperblock](X, X_AUX, x_dim_1, percent_c)
            cuda.synchronize()

            return X_AUX

        def crossAC0002() -> cp.core.core.ndarray:
            """crossAC0002.

            :rtype: cp.core.core.ndarray
            """

            @cuda.jit
            def kernel(
                X_AUX: cp.core.core.ndarray,
                C_M: cp.core.core.ndarray,
                digits: int,
                repetitions: int,
                x_dim_1: int,
                percent_c: float,
            ) -> None:
                """kernel.

                :param X_AUX:
                :type X_AUX: cp.core.core.ndarray
                :param C_M:
                :type C_M: cp.core.core.ndarray
                :param digits:
                :type digits: int
                :param repetitions:
                :type repetitions: int
                :param x_dim_1:
                :type x_dim_1: int
                :param percent_c:
                :type percent_c: float
                :rtype: None
                """
                row = cuda.grid(1)
                if row < n_samples:
                    if row % 2 == 0:
                        for i in range(digits):
                            count = 0
                            for j in range(0, int(math.ceil(x_dim_1 * percent_c)), 1):
                                if X_AUX[row, j] == i:
                                    count = count + 1
                                C_M[row, i] = count

                    else:
                        for i in range(digits):
                            count = 0
                            for j in range(
                                int(math.ceil(x_dim_1 * percent_c)), x_dim_1, 1
                            ):
                                if X_AUX[row, j] == i:
                                    count = count + 1
                                C_M[row, i] = count

            C_M = cp.zeros([n_samples, digits], dtype=cp.float32)
            x_dim_1 = digits * repetitions

            threadsperblock = 16
            blockspergrid_x = int(math.ceil((n_samples) / threadsperblock))
            blockspergrid = blockspergrid_x

            cuda.synchronize()
            kernel[blockspergrid, threadsperblock](
                X_AUX, C_M, digits, repetitions, x_dim_1, percent_c
            )
            cuda.synchronize()

            return C_M

        def crossAC0003() -> cp.core.core.ndarray:
            """crossAC0003.

            :rtype: cp.core.core.ndarray
            """

            @cuda.jit
            def kernel(
                X: cp.core.core.ndarray,
                X_AUX: cp.core.core.ndarray,
                C_M: cp.core.core.ndarray,
                digits: int,
                repetitions: int,
                x_dim_1: int,
                percent_c: float,
            ):
                """kernel.

                :param X:
                :type X: cp.core.core.ndarray
                :param X_AUX:
                :type X_AUX: cp.core.core.ndarray
                :param C_M:
                :type C_M: cp.core.core.ndarray
                :param digits:
                :type digits: int
                :param repetitions:
                :type repetitions: int
                :param x_dim_1:
                :type x_dim_1: int
                :param percent_c:
                :type percent_c: float
                """
                row = cuda.grid(1)
                if row < n_samples:
                    if row % 2 == 0:
                        discount = x_dim_1 - 1
                        for i in range(x_dim_1 - 1, -1, -1):
                            if discount >= int(math.ceil(x_dim_1 * percent_c)):
                                if (
                                    C_M[row, int(math.ceil(X[row + 1, i]))]
                                    < repetitions
                                ):
                                    X_AUX[row, discount] = X[row + 1, i]
                                    C_M[row, int(X[row + 1, i])] += 1
                                    discount -= 1

                    else:
                        count = 0
                        for i in range(0, x_dim_1, 1):
                            if count < int(math.ceil(x_dim_1 * percent_c)):
                                if (
                                    C_M[row, int(math.ceil(X[row - 1, i]))]
                                    < repetitions
                                ):
                                    X_AUX[row, count] = X[row - 1, i]
                                    C_M[row, int(X[row - 1, i])] += 1
                                    count += 1

            x_dim_1 = digits * repetitions

            threadsperblock = 16
            blockspergrid_x = int(math.ceil((n_samples) / threadsperblock))
            blockspergrid = blockspergrid_x

            cuda.synchronize()
            kernel[blockspergrid, threadsperblock](
                X, X_AUX, C_M, digits, repetitions, x_dim_1, percent_c
            )
            cuda.synchronize()

            return X_AUX

        X_AUX = crossAC0001()
        C_M = crossAC0002()
        X_AUX = crossAC0003()

        return X_AUX



from setuptools import setup  # type: ignore

with open("README.rst", "r") as readme_file:
    readme = readme_file.read()


setup(
    name="gepapy",
    version="1.1.6",
    description="Developing a library with Cuda and Python for solving scheduling problems on GPU",
    long_description=readme,
    author="Jean Carlo Jimenez Giraldo",
    author_email="mandalarotation@gmail.com",
    license="MIT",
    url="https://github.com/mandalarotation/GenSchedulingCuda-GSC",
    package_data={
        "gepapy": [
            "py.typed",
            "single_machine.pyi",
            "flow_shop.pyi",
            "job_shop.pyi",
            "operations.pyi",
        ],
        "gepapy.kernels": [
            "py.typed",
            "crossA0001.pyi",
            "fitnessA0001.pyi",
            "fitnessSM0001.pyi",
            "mutationA0001.pyi",
            "permutationA0001.pyi",
        ],
    },
    packages=[
        "gepapy",
        "gepapy.kernels",
        "gepapy.exceptions",
    ],
    scripts=[],
    install_requires=["numpy", "cupy", "numba", "typing"],
)



from __future__ import division
from numba import cuda  # type: ignore
import cupy as cp  # type: ignore
import math


class MutationA0001:
    """MutationA0001."""

    def __init__(self) -> None:
        """__init__.

        :rtype: None
        """
        pass

    def _mutationA0001(
        self,
        X: cp.core.core.ndarray,
        digits: int,
        repetitions: int,
        n_samples: int,
        percent_m: float,
    ) -> cp.core.core.ndarray:
        """_mutationA0001.

        :param X:
        :type X: cp.core.core.ndarray
        :param digits:
        :type digits: int
        :param repetitions:
        :type repetitions: int
        :param n_samples:
        :type n_samples: int
        :param percent_m:
        :type percent_m: float
        """

        def mutationAC0001() -> cp.core.core.ndarray:
            """mutationAC0001.

            :rtype: cp.core.core.ndarray
            """

            @cuda.jit
            def kernel(
                X_AUX: cp.core.core.ndarray,
                AL: cp.core.core.ndarray,
                digits: int,
                repetitions: int,
                n_samples: int,
            ):
                """kernel.

                :param X_AUX:
                :type X_AUX: cp.core.core.ndarray
                :param AL:
                :type AL: cp.core.core.ndarray
                :param digits:
                :type digits: int
                :param repetitions:
                :type repetitions: int
                :param n_samples:
                :type n_samples: int
                """
                row = cuda.grid(1)
                if row < n_samples:
                    for i in range(AL.shape[1]):
                        x_aux = X_AUX[row, int(math.ceil(AL[row, i, 0]))]
                        X_AUX[row, int(math.ceil(AL[row, i, 0]))] = X_AUX[
                            row, int(math.ceil(AL[row, i, 1]))
                        ]
                        X_AUX[row, int(math.ceil(AL[row, i, 1]))] = x_aux

            x_dim_1 = digits * repetitions

            X_AUX = cp.copy(X)

            AL = cp.array(
                cp.random.rand(n_samples, int(x_dim_1 * percent_m), 2), dtype=cp.float32
            ) * (x_dim_1 - 1)

            threadsperblock = 16
            blockspergrid_x = int(math.ceil((n_samples) / threadsperblock))
            blockspergrid = blockspergrid_x

            cuda.synchronize()
            kernel[blockspergrid, threadsperblock](
                X_AUX, AL, digits, repetitions, n_samples
            )
            cuda.synchronize()

            return X_AUX

        return mutationAC0001()



import cupy as cp  # type: ignore

class FitnessA0001:
    def __init__(self) -> None: ...
    def _fitnessA0001(
        self,
        x: cp.core.core.ndarray,
        d: cp.core.core.ndarray,
        w: cp.core.core.ndarray,
        T: cp.core.core.ndarray,
        M: cp.core.core.ndarray,
        digits: int,
        n_samples: int,
        n_machines: int,
    ) -> cp.core.core.ndarray: ...
    def _get_planA0001(
        self,
        row: int,
        X: cp.core.core.ndarray,
        T: cp.core.core.ndarray,
        M: cp.core.core.ndarray,
        digits: int,
        n_machines: int,
        fact_conv: float,
        start_time: int,
    ) -> list: ...
